@@ -27,6 +27,7 @@
 
 #include "terminal-intl.h"
 
+#include "terminal-debug.h"
 #include "terminal-app.h"
 #include "terminal-accels.h"
 #include "terminal-screen.h"
@@ -137,6 +138,12 @@ enum
   NUM_COLUMNS
 };
 
+enum
+{
+  SOURCE_DEFAULT = 0,
+  SOURCE_SESSION = 1
+};
+
 static TerminalApp *global_app = NULL;
 
 #define MONOSPACE_FONT_DIR "/desktop/gnome/interface"
@@ -1104,6 +1111,7 @@ new_profile_response_cb (GtkWidget *new_profile_dialog,
                                                    GTK_MESSAGE_QUESTION,
                                                    GTK_BUTTONS_YES_NO,
                                                    _("You already have a profile called “%s”. Do you want to create another profile with the same name?"), name);
+          /* Alternative button order was set automatically by GtkMessageDialog */
           retval = gtk_dialog_run (GTK_DIALOG (confirm_dialog));
           gtk_widget_destroy (confirm_dialog);
           if (retval == GTK_RESPONSE_NO)
@@ -1208,6 +1216,10 @@ terminal_app_new_profile (TerminalApp     *app,
 
       gtk_label_set_mnemonic_widget (GTK_LABEL (base_label), combo);
 
+      gtk_dialog_set_alternative_button_order (GTK_DIALOG (app->new_profile_dialog),
+                                               GTK_RESPONSE_ACCEPT,
+                                               GTK_RESPONSE_CANCEL,
+                                               -1);
       gtk_dialog_set_default_response (GTK_DIALOG (app->new_profile_dialog), GTK_RESPONSE_ACCEPT);
       gtk_dialog_set_response_sensitive (GTK_DIALOG (app->new_profile_dialog), GTK_RESPONSE_ACCEPT, FALSE);
     }
@@ -1643,19 +1655,41 @@ terminal_app_get (void)
   return global_app;
 }
 
+/**
+ * terminal_app_handle_options:
+ * @app:
+ * @options: a #TerminalOptions
+ * @allow_resume: whether to merge the terminal configuration from the
+ *   saved session on resume
+ * @error: a #GError to fill in
+ *
+ * Processes @options. It loads or saves the terminal configuration, or
+ * opens the specified windows and tabs.
+ *
+ * Returns: %TRUE if @options could be successfully handled, or %FALSE on
+ *   error
+ */
 gboolean
 terminal_app_handle_options (TerminalApp *app,
                              TerminalOptions *options,
+                             gboolean allow_resume,
                              GError **error)
 {
   GList *lw;
-  GdkScreen *screen;
+  GdkScreen *gdk_screen;
 
-  screen = terminal_app_get_screen_by_display_name (options->display_name,
-                                                    options->screen_number);
+  gdk_screen = terminal_app_get_screen_by_display_name (options->display_name,
+                                                        options->screen_number);
 
   if (options->save_config)
-    return terminal_app_save_config_file (app, options->config_file, error);
+    {
+      if (options->remote_arguments)
+        return terminal_app_save_config_file (app, options->config_file, error);
+      
+      g_set_error_literal (error, TERMINAL_OPTION_ERROR, TERMINAL_OPTION_ERROR_NOT_IN_FACTORY,
+                            "Cannot use \"--save-config\" when starting the factory process");
+      return FALSE;
+    }
 
   if (options->load_config)
     {
@@ -1664,7 +1698,7 @@ terminal_app_handle_options (TerminalApp *app,
 
       key_file = g_key_file_new ();
       result = g_key_file_load_from_file (key_file, options->config_file, 0, error) &&
-               terminal_options_merge_config (options, key_file, error);
+               terminal_options_merge_config (options, key_file, SOURCE_DEFAULT, error);
       g_key_file_free (key_file);
 
       if (!result)
@@ -1673,6 +1707,32 @@ terminal_app_handle_options (TerminalApp *app,
       /* fall-through on success */
     }
 
+#ifdef WITH_SMCLIENT
+{
+  EggSMClient *sm_client;
+
+  sm_client = egg_sm_client_get ();
+
+  if (allow_resume && egg_sm_client_is_resumed (sm_client))
+    {
+      GKeyFile *key_file;
+
+      key_file = egg_sm_client_get_state_file (sm_client);
+      if (key_file != NULL &&
+          !terminal_options_merge_config (options, key_file, SOURCE_SESSION, error))
+        return FALSE;
+    }
+}
+#endif
+
+  /* Make sure we open at least one window */
+  terminal_options_ensure_window (options);
+
+  if (options->startup_id != NULL)
+    _terminal_debug_print (TERMINAL_DEBUG_FACTORY,
+                           "Startup ID is '%s'\n",
+                           options->startup_id);
+
   for (lw = options->initial_windows;  lw != NULL; lw = lw->next)
     {
       InitialWindow *iw = lw->data;
@@ -1682,10 +1742,14 @@ terminal_app_handle_options (TerminalApp *app,
       g_assert (iw->tabs);
 
       /* Create & setup new window */
-      window = terminal_app_new_window (app, screen);
+      window = terminal_app_new_window (app, gdk_screen);
 
-      if (options->startup_id)
-        terminal_window_set_startup_id (window, options->startup_id);
+      /* Restored windows shouldn't demand attention; see bug #586308. */
+      if (iw->source_tag == SOURCE_SESSION)
+        terminal_window_set_is_restored (window);
+
+      if (options->startup_id != NULL)
+        gtk_window_set_startup_id (GTK_WINDOW (window), options->startup_id);
 
       /* Overwrite the default, unique window role set in terminal_window_init */
       if (iw->role)
@@ -1746,6 +1810,10 @@ terminal_app_handle_options (TerminalApp *app,
 
       if (iw->geometry)
         {
+          _terminal_debug_print (TERMINAL_DEBUG_GEOMETRY,
+                                "[window %p] applying geometry %s\n",
+                                window, iw->geometry);
+
           if (!gtk_window_parse_geometry (GTK_WINDOW (window), iw->geometry))
             g_printerr (_("Invalid geometry string \"%s\"\n"), iw->geometry);
         }
@@ -1796,8 +1864,6 @@ terminal_app_new_terminal (TerminalApp     *app,
   terminal_window_switch_screen (window, screen);
   gtk_widget_grab_focus (GTK_WIDGET (screen));
 
-  terminal_screen_launch_child (screen);
-
   return screen;
 }
 
@@ -1826,6 +1892,9 @@ terminal_app_edit_encodings (TerminalApp     *app,
 TerminalWindow *
 terminal_app_get_current_window (TerminalApp *app)
 {
+  if (app->windows == NULL)
+    return NULL;
+
   return g_list_last (app->windows)->data;
 }
 
@@ -1963,8 +2032,8 @@ terminal_app_save_config (TerminalApp *app,
       terminal_window_save_state (window, key_file, group);
     }
 
-  g_ptr_array_add (window_names_array, NULL);
   len = window_names_array->len;
+  g_ptr_array_add (window_names_array, NULL);
   window_names = (char **) g_ptr_array_free (window_names_array, FALSE);
   g_key_file_set_string_list (key_file, TERMINAL_CONFIG_GROUP, TERMINAL_CONFIG_PROP_WINDOWS, (const char * const *) window_names, len);
   g_strfreev (window_names);
